#!/usr/bin/env python3

def readCmdLineParameters():
	from argparse import ArgumentParser, SUPPRESS
	parser = ArgumentParser(add_help=False)
	required = parser.add_argument_group('required parameters')
	optional = parser.add_argument_group('optional parameters')
	required.add_argument('-o', '--output-file', help='output XLSM file name', required=True)
	required.add_argument('-i', '--input-file', help='input CSV file name', required=True)
	optional.add_argument('--csv-file', help='use custom CSV file')
	optional.add_argument('--vba-file', help='use custom VBA file')
	optional.add_argument('-h', '--help', action='help', default=SUPPRESS, help='show this help message and exit')
	readCmdLineParameters.args = parser.parse_args()

def main():
	import configparser, csv, os.path, subprocess, sys, xlsxwriter

	readCmdLineParameters()
	args = readCmdLineParameters.args
	
	script_name =  os.path.basename(sys.argv[0])
	budget_file = args.output_file
	input_csv_file = args.input_file
	iniFile = f'{sys.path[0]}/gen_budgetsheet.ini'
	exit_code = 0
	if args.csv_file:
		costsFile = args.csv_file
	else:
		costsFile = f'{sys.path[0]}/costs.csv'
		
	if args.vba_file:
		vbaFile = args.vba_file
	else:
		vbaFile = f'{sys.path[0]}/vbaProject.bin'
		
	if not os.path.isfile(input_csv_file):
		print(f'>> {script_name} ERROR: input file {input_csv_file} not found', file=sys.stderr)
		exit(2)

	if not os.path.isfile(iniFile):
		print(f'>> {script_name} ERROR: ini file {iniFile} not found', file=sys.stderr)
		exit(2)

	if not os.path.isfile(costsFile):
		print(f'>> {script_name} ERROR: costs file {costsFile} not found', file=sys.stderr)
		exit(2)

	if not os.path.isfile(vbaFile):
		print(f'>> {script_name} ERROR: VBA file {vbaFile} not found', file=sys.stderr)
		exit(2)

	# Create a workbook and add worksheets.
	workbook = xlsxwriter.Workbook(budget_file)
	worksheet1 = workbook.add_worksheet('Inputs')
	worksheet2 = workbook.add_worksheet('OLE Budget')
	worksheet3 = workbook.add_worksheet('PM and Combined Budgets')
	worksheet4 = workbook.add_worksheet('Budget Summary')

	# Macro
	workbook.add_vba_project(vbaFile)
	# these input_values should be confimed in VB Editor
	workbook.set_vba_name('ThisWorkbook')
	worksheet1.set_vba_name('Sheet1')
	worksheet2.set_vba_name('Sheet4')
	worksheet3.set_vba_name('Sheet2')
	worksheet4.set_vba_name('Sheet3')
										 
	# formats
	text_blue_right = workbook.add_format({'align': 'right', 'valign': 'vcenter', 'font_color': 'blue'})
	text_purple = workbook.add_format({'align': 'left', 'valign': 'vcenter', 'fg_color': '#CCCCFF', 'border': 1, 'border_color': '#A5A5A5'}) #CCCCFF - purple
	text_red = workbook.add_format({'align': 'left', 'valign': 'vcenter', 'font_color': 'red'})
	text_red_bold = workbook.add_format({'align': 'left', 'valign': 'vcenter', 'font_color': 'red', 'bold': True})
	text_white = workbook.add_format({'align': 'left', 'valign': 'vcenter'})
	number_green = workbook.add_format({'align': 'center', 'valign': 'vcenter', 'fg_color': '#92D050', 'border': 1, 'border_color': '#A5A5A5'}) #92D050 - green
	number_ochre = workbook.add_format({'align': 'center', 'valign': 'vcenter', 'fg_color': '#FFCC99', 'border': 1, 'border_color': '#A5A5A5'}) #FFCC99 - ochre
	money_format = workbook.add_format({'align': 'center', 'valign': 'vcenter', 'fg_color': '#FFCC99', 'num_format': '"$"#,##0.00', 'border': 1, 'border_color': '#A5A5A5'})
	center_format = workbook.add_format({'align': 'center', 'valign': 'vcenter'})
	bold_center = workbook.add_format({'align': 'center', 'valign': 'vcenter', 'bold': True, 'text_wrap': True})
	white_on_black = workbook.add_format({'align': 'center', 'valign': 'vcenter', 'fg_color': 'black', 'font_color': 'white', 'bold': True, 'text_wrap': True, 'border': 1, 'border_color': '#A5A5A5'})

	# Writing header
	# Text labels
	worksheet1.write_string('D1', 'Qty', bold_center)
	worksheet1.write_string('E1', 'COSTS', bold_center)
	worksheet1.write_string('F1', 'Price', bold_center)
	worksheet1.write_string('G1', 'Days Needed', bold_center)
	worksheet1.write_string('H1', 'If RTL', bold_center)

	worksheet1.write_string('C2', 'Average Word Count (per lang)')
	worksheet1.write_string('C3', 'Web engineering + QA (per lang)')
	worksheet1.write_string('C4', 'PLT (per lang)')
	worksheet1.write_string('C5', 'Graphics (# per lang)')
	worksheet1.write_string('C6', 'Graphics (Word Count per lang)')
	worksheet1.write_string('C7', 'PM (per lang)')

	worksheet1.merge_range('G8:I8', 'Standard Costs', white_on_black)
	worksheet1.write_row('A9',['TYPE', 'SCOPE', 'LANGUAGE', 'NEW WORDS', 'FUZZY WORDS', '100%', 'New words', 'Fuzzy', '100%'], white_on_black)

	# Formulae
	worksheet1.write_formula('D2', '=SUM(D10:F40)/SUM(B10:B40)', center_format)
	worksheet1.write_formula('G2', '=IF(D2/2000>1,CEILING(D2/2000,1),1)', center_format)
	worksheet1.write_formula('D3', '=IF(D2>375,ROUND(D2/375,0),3)', center_format)
	worksheet1.write_formula('G3', '=IF(D3/6>2,CEILING(D3/6,1),2)', center_format)
	worksheet1.write_formula('H3', '=G3*2', center_format)
	worksheet1.write_formula('D4', '=IF(ROUND((D2/15000)*4,0),ROUND((D2/15000)*8,0),1)', center_format)
	worksheet1.write_formula('G4', '=IF(D4/6>2,CEILING(D4/6,1),2)', center_format)
	worksheet1.write_formula('H4', '=G4*2', center_format)
	worksheet1.write_formula('D7', '=IF(D2>4000,ROUND(0.1*(1500*0.24+D3*E3+D4*E4+D5*E5)/30,0),1)', center_format)

	worksheet1.write_number('D5', 0, center_format)
	worksheet1.write_number('D6', 0, center_format)

	configParser = configparser.RawConfigParser()   
	configParser.read(iniFile)

	worksheet1.write_number('E3', configParser.getint('costs', 'eng'), money_format)
	worksheet1.write_number('E4', configParser.getint('costs', 'plt'), money_format)
	worksheet1.write_number('E5', configParser.getint('costs', 'dtp'), money_format)
	worksheet1.write_number('E7', configParser.getint('costs', 'pm'), money_format)

	worksheet1.write_number('F3', configParser.getint('prices', 'eng'), money_format)
	worksheet1.write_number('F4', configParser.getint('prices', 'plt'), money_format)
	worksheet1.write_number('F5', configParser.getint('prices', 'dtp'), money_format)
	worksheet1.write_number('F7', configParser.getint('prices', 'pm'), money_format)

	# Width
	worksheet1.set_column('A:I', 8)
	worksheet1.set_column('C:C', 30)
	worksheet1.set_column('G:G', 13)

	# Writing input data
	# read costs
	with open(costsFile) as f:
		costs = list(csv.reader(f))

	for i in range(len(costs)):
		if costs[i][0] == "Default":
			default_index = i
			break

	with open(input_csv_file) as f:
		inputs = list(csv.reader(f))

	text_format = text_white # Let's start with purple

	for row_number, row in enumerate(inputs):
		# white-pulple interchange
		if text_format == text_white:
			text_format = text_purple
		else:
			text_format = text_white
		
		for column_number, input_value in enumerate(row):
			# Cells with text should have text format
			if column_number == 1 or column_number == 3:
				table_format = number_green
			elif column_number == 0 or column_number == 2:
				table_format = text_format
			else:
				table_format = number_ochre
			
			# We need to write numbers if possible (inportant for formulae)
			try:
				input_value = float(input_value)
				worksheet1.write_number(row_number+9, column_number, input_value, table_format)#, center_format)
			except ValueError:
				worksheet1.write_string(row_number+9, column_number, input_value, table_format)#, center_format)
		# Add costs
		index = 0
		for i in range(len(costs)):
			if costs[i][0] == inputs[row_number][2]:
				index = i
				break
		else:
			index = default_index
			print(f'>> {script_name} WARNING: missing language! "{inputs[row_number][2]}" does not exist in {costsFile}')
			exit_code = 99

		for j in range(1, len(costs[index])):
			input_value = costs[index][j]
			try:
				input_value = float(input_value)
				worksheet1.write_number(row_number+9, column_number+j, input_value, money_format)
			except ValueError:
				worksheet1.write_string(row_number+9, column_number+j, input_value, money_format)
		
		if index == default_index:
			worksheet1.write_string(row_number+9, 9, 'WARNING: Rates not found for this locale. These rates are the default values. Please review and supply the rates to the OLE to prevent errors in the future.', text_red_bold)
		
	worksheet2.insert_button('B1', {'macro': 'BuildbudgetOLE', 'caption': 'Generate OLE Budget', 'height': 40})
	worksheet3.insert_button('B1', {'macro': 'Buildbudget', 'caption': 'Generate Budget', 'height': 40})

	#static fields on sheet2
	worksheet2.write_string('M2', 'This is the OLE word count per hour table:')
	worksheet2.write_row('M3', [1, 1])
	worksheet2.write_row('M4', [1499, 1.5])
	worksheet2.write_row('M5', [2499, 2])
	worksheet2.write_row('M6', [4499, 3])
	worksheet2.write_row('M7', [5999, 4])
	worksheet2.write_row('M8', [8999, 5])
	worksheet2.write_row('M9', [11999, 6])
	worksheet2.write_row('M10', [15999, 7])
	worksheet2.write_row('M11', [19999, 8])
	worksheet2.write_row('M12', [24999, 9])
	worksheet2.write_row('M13', [30000, 10])
	worksheet2.write_row('M14', [100000, 12])
	worksheet2.write_string('M16', "You can change it, but don't MOVE it.")

	# column width on sheets 2-4
	worksheet2.set_column('B:B', 25)
	worksheet2.set_column('C:C', 44)
	worksheet3.set_column('B:B', 25)
	worksheet3.set_column('C:C', 44)
	worksheet4.set_column('B:B', 25)

	# text on sheet 4
	worksheet4.write_string('B12', 'Note:', text_blue_right)
	worksheet4.write_string('C12','Summary assumes PSDs are available for DTP tasks.')
	worksheet4.write_rich_string('C13','If PSDs are not available, change cell C7 to total in ', text_red, 'No PSD', ' column on previous tab')

	workbook.close()

	exit(exit_code)

if __name__ == "__main__":
    main()
